package com.landray.kmss.sys.attachment.io;

import java.io.InputStream;

public class DecryptionInputStream extends AbstractInputStream
{
    private boolean noEncryption;
    private boolean isAESEncryption;
    private boolean isSM4Encryption;
    
    public DecryptionInputStream(final InputStream originStream) throws Exception {
        super(originStream);
    }
    
    public DecryptionInputStream(final InputStream originStream, final int encryptionMode) throws Exception {
        super(originStream, encryptionMode);
    }
    
    @Override
    protected void init() throws Exception {
        this.noEncryption = (this.cryptionMode == 0);
        if (this.noEncryption) {
            return;
        }
        if (this.cryptionMode != 0) {
            this.initAES();
            this.initSM4();
        }
        final int len = DecryptionInputStream.markBytes.length;
        final byte[] b = new byte[len];
        final int count = this.originStream.read(b);
        if (count > 0) {
            this.isSM4Encryption = this.isSM4Encryption(b);
            if (this.isSM4Encryption) {
                return;
            }
            this.isAESEncryption = this.isAESEncryption(b);
            if (this.isAESEncryption) {
                return;
            }
            for (int i = 0; i < len; ++i) {
                if (this.format(b[i] + 128) != DecryptionInputStream.markBytes[i] + 128) {
                    this.additionalBytes = b;
                    this.noEncryption = true;
                    return;
                }
            }
        }
    }
    
    @Override
    protected void format(final byte[] b, final int off, final int len) {
        if (this.noEncryption) {
            return;
        }
        for (int i = off; i < len; ++i) {
            if (this.isAESEncryption) {
                b[i] = this.aesCrypt.decrypt(b, i, 1)[0];
            }
            else if (this.isSM4Encryption) {
                b[i] = this.sm4Crypt.decrypt(b, i, 1)[0];
            }
            else if (b[i] == -128) {
                b[i] = 127;
            }
            else {
                final int n = i;
                --b[n];
            }
        }
    }
    
    @Override
    protected int format(final int i) {
        if (this.noEncryption) {
            return i;
        }
        if (i == -1) {
            return -1;
        }
        if (this.isAESEncryption) {
            final byte[] b = this.aesCrypt.decrypt(new byte[] { (byte)i });
            return b[0];
        }
        if (this.isSM4Encryption) {
            final byte[] b = this.sm4Crypt.decrypt(new byte[] { (byte)i });
            return b[0];
        }
        if (i == 0) {
            return 255;
        }
        return i - 1;
    }
    
    public boolean isAESEncryption() {
        return this.isAESEncryption;
    }
    
    public boolean isSM4Encryption() {
        return this.isSM4Encryption;
    }
}
